library(ggplot2)
setwd("Q:/SeventhPlan/RPM - Model/Technical Appendix/")
ngas <- read.csv("NGasForecast.csv", header=TRUE)
ngas$Factor <- ngas$Price/rep(ngas$Price[ngas$Forecast=="Med"], 3)
ngas.plot <- ggplot(ngas, aes(x=Period, y=Price, group=Forecast, colour=Forecast)) + geom_line()
dev.new()
ngas.plot
ngasfact.plot <- ggplot(ngas, aes(x=Period, y=Factor, group=Forecast, colour=Forecast)) + geom_line()
dev.new()
ngasfact.plot 

ngas <- ngas[ngas$Zone == "West", ]
ngas.wide <- reshape(ngas, idvar=c("Period", "Zone"), timevar="Forecast", direction="wide")
ngas.wide$Year <- c(0, rep(1:19, each=4), rep(20, 3))
 
high.mod <- lm(log(Price.High/Price.Med)~Year+I(Year^2), data=ngas.wide)
low.mod <- lm(log(Price.Low/Price.Med)~Year+I(Year^2), data=ngas.wide)
high.coef <- high.mod$coef
low.coef <- low.mod$coef
high.rsq <- summary(high.mod)$r.squared 
low.rsq <- summary(low.mod)$r.squared 

#Per conversation with Massoud, using the fit for the upper forecast
rsq.weight <- c(high.rsq, low.rsq)/(high.rsq + low.rsq)
coef.weight <- high.coef * 1 - low.coef * 0

#Now trying to estimate seasonal factors using the history
hist <- read.csv("NGasHist.csv", header=TRUE, stringsAsFactors=FALSE)
hist$Date <- as.Date(hist$Date)
hist$Quarter <- floor((as.numeric(format(hist$Date, "%m"))-1)/3 + 1)
hist$Year <- format(hist$Date, "%Y")
qhist <- aggregate(Price~Quarter+Year, data=hist, FUN=mean)
annual.hist <- aggregate(Price~Year, data=hist, FUN=mean)
names(annual.hist) <- c("Year", "AnnPrice")
qhist <- merge(qhist, annual.hist)
qhist$Factor <- qhist$Price/qhist$AnnPrice
qhistmean <- aggregate(Factor ~ Quarter, qhist, mean)
names(qhistmean) <- c("Quarter", "AvgQFactor")
qhist <- merge(qhist, qhistmean)
qhist$NormalizedFactor <- qhist$Factor/qhist$AvgQFactor 
qhist$LogNormalizedFactor <- log(qhist$NormalizedFactor)
qsd <- aggregate(LogNormalizedFactor ~ Quarter, data=qhist, sd)

#What is the biggest historic quarterly jump?
log(max(qhist$Factor))
log(min(qhist$Factor))

#According to Massoud we're trying to hit the high and low forecasts at 80% and 10% respectively
generate.games <- function(x){
  year <- c(0, rep(1:19, each=4), rep(20, 3)) 
  (coef.weight/qnorm(.80)*rnorm(3))%*% t(matrix(c(rep(1,80), year, year^2), nrow=80, ncol=3))
}
games <- sapply(1:800, generate.games)
games.df <- as.data.frame(exp(games)*ngas.wide$Price.Med)
games.df$period <- 1:80

long.game.df <- reshape(games.df, idvar="period", varying=list(1:800), direction="long")
game.plot <- ggplot(subset(long.game.df, time%in%1:10), aes(x=period, y=V1, group=time, colour="Simultation")) + geom_line()
game.plot <- game.plot + geom_line(data=ngas.wide, aes(x=Period, y=Price.Low, group="LOW", colour="Low"), lwd=1)
game.plot <- game.plot + geom_line(data=ngas.wide, aes(x=Period, y=Price.High, group="HIGH", colour="High"), lwd=1)
game.plot <- game.plot + ylab("Price ($/MMBTU)") + theme(legend.title=element_blank())
dev.new()
game.plot

sum(games.df[80, -c(801)] > ngas.wide$Price.High[80])
sum(games.df[80, -c(801)] > ngas.wide$Price.High[80])/800

#Here is seasonal SD beyond annual shapes as applied above
qsd$LogNormalizedFactor
generate.seasonal <- function(x, q.sds) {
  exp(rnorm(n=80, mean=rep(0, 4), sd=q.sds[c(4,1,2,3)]))
}
seasonal <- sapply(1:800, generate.seasonal, q.sds=qsd$LogNormalizedFactor)
games.df <- games.df[,-c(801)]*seasonal

long.game.df <- reshape(games.df, idvar="period", varying=list(1:800), direction="long")
game.plot <- ggplot(subset(long.game.df, time%in%1:10), aes(x=period, y=V1, group=time, colour="Simulation")) + geom_line()
game.plot <- game.plot + geom_line(data=ngas.wide, aes(x=Period, y=Price.Low, group="LOW", colour="Low"), lwd=1)
game.plot <- game.plot + geom_line(data=ngas.wide, aes(x=Period, y=Price.High, group="HIGH", colour="High"), lwd=1)
game.plot <- game.plot + ylab("Price ($/MMBTU)") + theme(legend.title=element_blank())
dev.new()
game.plot

#If all that looks good here are the coefficients
coef.weight


